@@ -1,4 +1,4 @@
-/*	$OpenBSD: util.c,v 1.62 2022/02/06 09:51:19 claudio Exp $ */
+/*	$OpenBSD: util.c,v 1.77 2023/04/17 08:02:21 claudio Exp $ */
 
 /*
  * Copyright (c) 2006 Claudio Jeker <claudio@openbsd.org>
@@ -60,14 +60,16 @@ log_in6addr(const struct in6_addr *addr)
 {
 	struct sockaddr_in6	sa_in6;
 
-	bzero(&sa_in6, sizeof(sa_in6));
+	memset(&sa_in6, 0, sizeof(sa_in6));
 	sa_in6.sin6_family = AF_INET6;
 	memcpy(&sa_in6.sin6_addr, addr, sizeof(sa_in6.sin6_addr));
 
 #ifdef __KAME__
 	/* XXX thanks, KAME, for this ugliness... adopted from route/show.c */
-	if (IN6_IS_ADDR_LINKLOCAL(&sa_in6.sin6_addr) ||
-	    IN6_IS_ADDR_MC_LINKLOCAL(&sa_in6.sin6_addr)) {
+	if ((IN6_IS_ADDR_LINKLOCAL(&sa_in6.sin6_addr) ||
+	    IN6_IS_ADDR_MC_LINKLOCAL(&sa_in6.sin6_addr) ||
+	    IN6_IS_ADDR_MC_NODELOCAL(&sa_in6.sin6_addr)) &&
+	    sa_in6.sin6_scope_id == 0) {
 		uint16_t tmp16;
 		memcpy(&tmp16, &sa_in6.sin6_addr.s6_addr[2], sizeof(tmp16));
 		sa_in6.sin6_scope_id = ntohs(tmp16);
@@ -129,7 +131,9 @@ log_rd(uint64_t rd)
 		snprintf(buf, sizeof(buf), "rd %s:%hu", inet_ntoa(addr), u16);
 		break;
 	default:
-		return ("rd ?");
+		snprintf(buf, sizeof(buf), "rd #%016llx",
+		    (unsigned long long)rd);
+		break;
 	}
 	return (buf);
 }
@@ -139,7 +143,7 @@ const struct ext_comm_pairs iana_ext_comms[] = IANA_EXT_COMMUNITIES;
 /* NOTE: this function does not check if the type/subtype combo is
  * actually valid. */
 const char *
-log_ext_subtype(short type, uint8_t subtype)
+log_ext_subtype(int type, uint8_t subtype)
 {
 	static char etype[6];
 	const struct ext_comm_pairs *cp;
@@ -182,7 +186,7 @@ log_rtr_error(enum rtr_error err)
 	case UNSUPP_PDU_TYPE:
 		return "Unsupported PDU Type";
 	case UNK_REC_WDRAWL:
-		return "Withdrawl of Unknown Record";
+		return "Withdrawal of Unknown Record";
 	case DUP_REC_RECV:
 		return "Duplicate Announcement Received";
 	case UNEXP_PROTOCOL_VERS:
@@ -193,6 +197,25 @@ log_rtr_error(enum rtr_error err)
 	}
 }
 
+const char *
+log_policy(enum role role)
+{
+	switch (role) {
+	case ROLE_PROVIDER:
+		return "provider";
+	case ROLE_RS:
+		return "rs";
+	case ROLE_RS_CLIENT:
+		return "rs-client";
+	case ROLE_CUSTOMER:
+		return "customer";
+	case ROLE_PEER:
+		return "peer";
+	default:
+		return "unknown";
+	}
+}
+
 const char *
 aspath_delim(uint8_t seg_type, int closing)
 {
@@ -343,7 +366,7 @@ aspath_strlen(void *data, uint16_t len)
 /*
  * Extract the asnum out of the as segment at the specified position.
  * Direct access is not possible because of non-aligned reads.
- * ATTENTION: no bounds checks are done.
+ * Only works on verified 4-byte AS paths.
  */
 uint32_t
 aspath_extract(const void *seg, int pos)
@@ -351,6 +374,9 @@ aspath_extract(const void *seg, int pos)
 	const u_char	*ptr = seg;
 	uint32_t	 as;
 
+	/* minimal pos check, return 0 since that is an invalid ASN */
+	if (pos < 0 || pos >= ptr[1])
+		return (0);
 	ptr += 2 + sizeof(uint32_t) * pos;
 	memcpy(&as, ptr, sizeof(uint32_t));
 	return (ntohl(as));
@@ -468,26 +494,24 @@ aspath_inflate(void *data, uint16_t len, uint16_t *newlen)
 }
 
 /* NLRI functions to extract prefixes from the NLRI blobs */
-static int
-extract_prefix(u_char *p, uint16_t len, void *va,
-    uint8_t pfxlen, uint8_t max)
+int
+extract_prefix(const u_char *p, int len, void *va, uint8_t pfxlen, uint8_t max)
 {
 	static u_char	 addrmask[] = {
 	    0x00, 0x80, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc, 0xfe, 0xff };
 	u_char		*a = va;
-	int		 i;
-	uint16_t	 plen = 0;
+	int		 plen;
 
-	for (i = 0; pfxlen && i < max; i++) {
-		if (len <= plen)
-			return (-1);
+	plen = PREFIX_SIZE(pfxlen) - 1;
+	if (len < plen || max < plen)
+		return -1;
+
+	while (pfxlen > 0) {
 		if (pfxlen < 8) {
-			a[i] = *p++ & addrmask[pfxlen];
-			plen++;
+			*a++ = *p++ & addrmask[pfxlen];
 			break;
 		} else {
-			a[i] = *p++;
-			plen++;
+			*a++ = *p++;
 			pfxlen -= 8;
 		}
 	}
@@ -507,7 +531,7 @@ nlri_get_prefix(u_char *p, uint16_t len, struct bgpd_addr *prefix,
 	pfxlen = *p++;
 	len--;
 
-	bzero(prefix, sizeof(struct bgpd_addr));
+	memset(prefix, 0, sizeof(struct bgpd_addr));
 	prefix->aid = AID_INET;
 	*prefixlen = pfxlen;
 
@@ -533,7 +557,7 @@ nlri_get_prefix6(u_char *p, uint16_t len, struct bgpd_addr *prefix,
 	pfxlen = *p++;
 	len--;
 
-	bzero(prefix, sizeof(struct bgpd_addr));
+	memset(prefix, 0, sizeof(struct bgpd_addr));
 	prefix->aid = AID_INET6;
 	*prefixlen = pfxlen;
 
@@ -561,7 +585,7 @@ nlri_get_vpn4(u_char *p, uint16_t len, struct bgpd_addr *prefix,
 	p += 1;
 	plen = 1;
 
-	bzero(prefix, sizeof(struct bgpd_addr));
+	memset(prefix, 0, sizeof(struct bgpd_addr));
 
 	/* label stack */
 	do {
@@ -572,6 +596,7 @@ nlri_get_vpn4(u_char *p, uint16_t len, struct bgpd_addr *prefix,
 			return (-1);
 		if (withdraw) {
 			/* on withdraw ignore the labelstack all together */
+			p += 3;
 			plen += 3;
 			pfxlen -= 3 * 8;
 			break;
@@ -635,6 +660,7 @@ nlri_get_vpn6(u_char *p, uint16_t len, struct bgpd_addr *prefix,
 			return (-1);
 		if (withdraw) {
 			/* on withdraw ignore the labelstack all together */
+			p += 3;
 			plen += 3;
 			pfxlen -= 3 * 8;
 			break;
@@ -675,7 +701,14 @@ nlri_get_vpn6(u_char *p, uint16_t len, struct bgpd_addr *prefix,
 	return (plen + rv);
 }
 
+static in_addr_t
+prefixlen2mask(uint8_t prefixlen)
+{
+	if (prefixlen == 0)
+		return (0);
 
+	return (0xffffffff << (32 - prefixlen));
+}
 
 /*
  * This function will have undefined behaviour if the passed in prefixlen is
@@ -750,15 +783,6 @@ prefix_compare(const struct bgpd_addr *a, const struct bgpd_addr *b,
 
 }
 
-in_addr_t
-prefixlen2mask(uint8_t prefixlen)
-{
-	if (prefixlen == 0)
-		return (0);
-
-	return (0xffffffff << (32 - prefixlen));
-}
-
 void
 inet4applymask(struct in_addr *dest, const struct in_addr *src, int prefixlen)
 {
@@ -774,7 +798,7 @@ inet6applymask(struct in6_addr *dest, const struct in6_addr *src, int prefixlen)
 	struct in6_addr	mask;
 	int		i;
 
-	bzero(&mask, sizeof(mask));
+	memset(&mask, 0, sizeof(mask));
 	for (i = 0; i < prefixlen / 8; i++)
 		mask.s6_addr[i] = 0xff;
 	i = prefixlen % 8;
@@ -785,6 +809,22 @@ inet6applymask(struct in6_addr *dest, const struct in6_addr *src, int prefixlen)
 		dest->s6_addr[i] = src->s6_addr[i] & mask.s6_addr[i];
 }
 
+void
+applymask(struct bgpd_addr *dest, const struct bgpd_addr *src, int prefixlen)
+{
+	*dest = *src;
+	switch (src->aid) {
+	case AID_INET:
+	case AID_VPN_IPv4:
+		inet4applymask(&dest->v4, &src->v4, prefixlen);
+		break;
+	case AID_INET6:
+	case AID_VPN_IPv6:
+		inet6applymask(&dest->v6, &src->v6, prefixlen);
+		break;
+	}
+}
+
 /* address family translation functions */
 const struct aid aid_vals[AID_MAX] = AID_VALS;
 
@@ -860,7 +900,7 @@ addr2sa(const struct bgpd_addr *addr, uint16_t port, socklen_t *len)
 	if (addr == NULL || addr->aid == AID_UNSPEC)
 		return (NULL);
 
-	bzero(&ss, sizeof(ss));
+	memset(&ss, 0, sizeof(ss));
 	switch (addr->aid) {
 	case AID_INET:
 	case AID_VPN_IPv4:
@@ -878,6 +918,9 @@ addr2sa(const struct bgpd_addr *addr, uint16_t port, socklen_t *len)
 		sa_in6->sin6_scope_id = addr->scope_id;
 		*len = sizeof(struct sockaddr_in6);
 		break;
+	case AID_FLOWSPECv4:
+	case AID_FLOWSPECv6:
+		return (NULL);
 	}
 
 	return ((struct sockaddr *)&ss);
@@ -889,7 +932,7 @@ sa2addr(struct sockaddr *sa, struct bgpd_addr *addr, uint16_t *port)
 	struct sockaddr_in		*sa_in = (struct sockaddr_in *)sa;
 	struct sockaddr_in6		*sa_in6 = (struct sockaddr_in6 *)sa;
 
-	bzero(addr, sizeof(*addr));
+	memset(addr, 0, sizeof(*addr));
 	switch (sa->sa_family) {
 	case AF_INET:
 		addr->aid = AID_INET;
@@ -899,24 +942,24 @@ sa2addr(struct sockaddr *sa, struct bgpd_addr *addr, uint16_t *port)
 		break;
 	case AF_INET6:
 		addr->aid = AID_INET6;
-		memcpy(&addr->v6, &sa_in6->sin6_addr, sizeof(addr->v6));
 #ifdef __KAME__
 		/*
 		 * XXX thanks, KAME, for this ugliness...
 		 * adopted from route/show.c
 		 */
-		if (IN6_IS_ADDR_LINKLOCAL(&sa_in6->sin6_addr) ||
-		    IN6_IS_ADDR_MC_LINKLOCAL(&sa_in6->sin6_addr)) {
+		if ((IN6_IS_ADDR_LINKLOCAL(&sa_in6->sin6_addr) ||
+		    IN6_IS_ADDR_MC_LINKLOCAL(&sa_in6->sin6_addr) ||
+		    IN6_IS_ADDR_MC_NODELOCAL(&sa_in6->sin6_addr)) &&
+		    sa_in6->sin6_scope_id == 0) {
 			uint16_t tmp16;
 			memcpy(&tmp16, &sa_in6->sin6_addr.s6_addr[2],
 			    sizeof(tmp16));
-			if (tmp16 != 0) {
-				sa_in6->sin6_scope_id = ntohs(tmp16);
-				sa_in6->sin6_addr.s6_addr[2] = 0;
-				sa_in6->sin6_addr.s6_addr[3] = 0;
-			}
+			sa_in6->sin6_scope_id = ntohs(tmp16);
+			sa_in6->sin6_addr.s6_addr[2] = 0;
+			sa_in6->sin6_addr.s6_addr[3] = 0;
 		}
 #endif
+		memcpy(&addr->v6, &sa_in6->sin6_addr, sizeof(addr->v6));
 		addr->scope_id = sa_in6->sin6_scope_id; /* I hate v6 */
 		if (port)
 			*port = ntohs(sa_in6->sin6_port);
