@@ -1,4 +1,4 @@
-/*	$OpenBSD: control.c,v 1.110 2023/04/20 12:53:27 claudio Exp $ */
+/*	$OpenBSD: control.c,v 1.113 2023/09/28 07:01:26 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Henning Brauer <henning@openbsd.org>
@@ -314,7 +314,6 @@ control_dispatch_msg(struct pollfd *pfd, struct peer_head *peers)
 			if (imsg.hdr.len == IMSG_HEADER_SIZE +
 			    sizeof(struct ctl_neighbor)) {
 				neighbor = imsg.data;
-				neighbor->descr[PEER_DESCR_LEN - 1] = 0;
 			} else {
 				neighbor = NULL;
 			}
@@ -370,7 +369,6 @@ control_dispatch_msg(struct pollfd *pfd, struct peer_head *peers)
 			}
 
 			neighbor = imsg.data;
-			neighbor->descr[PEER_DESCR_LEN - 1] = 0;
 
 			matched = 0;
 			RB_FOREACH(p, peer_head, peers) {
@@ -390,17 +388,23 @@ control_dispatch_msg(struct pollfd *pfd, struct peer_head *peers)
 					control_result(c, CTL_RES_OK);
 					break;
 				case IMSG_CTL_NEIGHBOR_DOWN:
-					p->conf.down = 1;
+					neighbor->reason[
+					    sizeof(neighbor->reason) - 1] =
+					    '\0';
 					strlcpy(p->conf.reason,
 					    neighbor->reason,
-					    sizeof(neighbor->reason));
+					    sizeof(p->conf.reason));
+					p->conf.down = 1;
 					session_stop(p, ERR_CEASE_ADMIN_DOWN);
 					control_result(c, CTL_RES_OK);
 					break;
 				case IMSG_CTL_NEIGHBOR_CLEAR:
+					neighbor->reason[
+					    sizeof(neighbor->reason) - 1] =
+					    '\0';
 					strlcpy(p->conf.reason,
 					    neighbor->reason,
-					    sizeof(neighbor->reason));
+					    sizeof(p->conf.reason));
 					p->IdleHoldTime =
 					    INTERVAL_IDLE_HOLD_INITIAL;
 					p->errcnt = 0;
@@ -474,7 +478,6 @@ control_dispatch_msg(struct pollfd *pfd, struct peer_head *peers)
 
 			ribreq = imsg.data;
 			neighbor = &ribreq->neighbor;
-			neighbor->descr[PEER_DESCR_LEN - 1] = 0;
 
 			/* check if at least one neighbor exists */
 			RB_FOREACH(p, peer_head, peers)
