@@ -1069,7 +1069,7 @@ kr_net_redist_add(struct ktable *kt, struct network_config *net,
 	}
 
 	if (send_network(IMSG_NETWORK_ADD, net, attr) == -1)
-		log_warnx("%s: faild to send network update", __func__);
+		log_warnx("%s: failed to send network update", __func__);
 	return 1;
 }
 
@@ -1095,7 +1095,7 @@ kr_net_redist_del(struct ktable *kt, struct network_config *net, int dynamic)
 	free(r);
 
 	if (send_network(IMSG_NETWORK_REMOVE, net, NULL) == -1)
-		log_warnx("%s: faild to send network removal", __func__);
+		log_warnx("%s: failed to send network removal", __func__);
 }
 
 int
