@@ -1,4 +1,4 @@
-/*	$OpenBSD: rde.h,v 1.294 2023/06/12 12:48:07 claudio Exp $ */
+/*	$OpenBSD: rde.h,v 1.296 2023/08/16 08:26:35 claudio Exp $ */
 
 /*
  * Copyright (c) 2003, 2004 Claudio Jeker <claudio@openbsd.org> and
@@ -108,10 +108,8 @@ struct rde_peer {
 
 struct rde_aspa;
 struct rde_aspa_state {
-	uint8_t		onlyup_v4;
-	uint8_t		downup_v4;
-	uint8_t		onlyup_v6;
-	uint8_t		downup_v6;
+	uint8_t		onlyup;
+	uint8_t		downup;
 };
 
 #define AS_SET			1
@@ -447,10 +445,7 @@ int	community_add(struct rde_community *, int, void *, size_t);
 int	community_large_add(struct rde_community *, int, void *, size_t);
 int	community_ext_add(struct rde_community *, int, int, void *, size_t);
 
-int	community_write(struct rde_community *, void *, uint16_t);
-int	community_large_write(struct rde_community *, void *, uint16_t);
-int	community_ext_write(struct rde_community *, int, void *, uint16_t);
-int	community_writebuf(struct ibuf *, struct rde_community *);
+int	community_writebuf(struct rde_community *, uint8_t, int, struct ibuf *);
 
 void			 communities_shutdown(void);
 struct rde_community	*communities_lookup(struct rde_community *);
@@ -519,8 +514,7 @@ struct pt_entry	*pt_add_flow(struct flowspec *);
 void	 pt_remove(struct pt_entry *);
 struct pt_entry	*pt_lookup(struct bgpd_addr *);
 int	 pt_prefix_cmp(const struct pt_entry *, const struct pt_entry *);
-int	 pt_write(u_char *, int, struct pt_entry *, int);
-int	 pt_writebuf(struct ibuf *, struct pt_entry *);
+int	 pt_writebuf(struct ibuf *, struct pt_entry *, int, int, uint32_t);
 
 static inline struct pt_entry *
 pt_ref(struct pt_entry *pt)
@@ -710,17 +704,15 @@ void		 up_generate_addpath_all(struct rde_peer *, struct rib_entry *,
 		    struct prefix *, struct prefix *);
 void		 up_generate_default(struct rde_peer *, uint8_t);
 int		 up_is_eor(struct rde_peer *, uint8_t);
-int		 up_dump_withdraws(u_char *, int, struct rde_peer *, uint8_t);
-int		 up_dump_mp_unreach(u_char *, int, struct rde_peer *, uint8_t);
-int		 up_dump_attrnlri(u_char *, int, struct rde_peer *);
-int		 up_dump_mp_reach(u_char *, int, struct rde_peer *, uint8_t);
+int		 up_dump_withdraws(struct ibuf *, struct rde_peer *, uint8_t);
+int		 up_dump_update(struct ibuf *, struct rde_peer *, uint8_t);
 
 /* rde_aspa.c */
 void		 aspa_validation(struct rde_aspa *, struct aspath *,
 		    struct rde_aspa_state *);
 struct rde_aspa	*aspa_table_prep(uint32_t, size_t);
 void		 aspa_add_set(struct rde_aspa *, uint32_t, const uint32_t *,
-		    uint32_t, const uint32_t *);
+		    uint32_t);
 void		 aspa_table_free(struct rde_aspa *);
 void		 aspa_table_stats(const struct rde_aspa *,
 		    struct ctl_show_set *);
